(function() {

    "use strict";

    /* globals angular */
    const yaml = require('js-yaml');
    const fs = require('fs');
    const path = require('path');
    var remote = require('electron').remote; // Load remote component that contains the dialog dependency
    //var dialog = remote.require('dialog'); // Load the dialogs component of the OS
    //const { dialog } = require('electron');
    const dialog = require('electron').remote.dialog;
    var moment = require('moment-timezone');
    const globule = require('globule');
    
    const ipcRenderer = require('electron').ipcRenderer;
    // var httpPort = 8888;
    var httpPort = 5364;
    ipcRenderer.on('setPortReply', function (event, newPort) {
        httpPort = newPort;
    });
    ipcRenderer.send('setPort');
    const util = require('util');

    let configDir = 'rmconnect';
    let configPath = path.join(process.env.HOME || process.env.USERPROFILE, '/'+configDir+'/config.yml');
    let logsPath = path.join(process.env.HOME || process.env.USERPROFILE, '/'+configDir+'/logs/');
    
    
    angular
        .module('rmApp')
        .controller('listController', listController);

    listController.$inject = ['files', '$scope', 'logs', 'logfiles', '$mdToast'];

    function listController(files, $scope, logs, logfiles, $mdToast) {
        /* jshint validthis: true */

        /**
         * Keeps the endpoints
         *
         * @var {Array}
         */
        $scope.endpoints = [
            '/api/v1/items/current/',
            '/api/v1/items/playlist/',
            '/api/v1/items/playlist/item',
        ];

        $scope.$watch(() => {
            return httpPort;
        }, () => {
            $scope.useHttpDomain = 'http://127.0.0.1:' + httpPort;
        });



        /* deprecated (eermm ,i think?)
        $scope.updateCurrent = function(){
            if ($scope.current){
                switch($scope.current.endpoint){
                    case '/api/v1/items/current/':
                        $scope.current.playlist = false;
                        $scope.current.multiplehours = false;
                        break;
                    case '/api/v1/items/playlist/':
                        $scope.current.playlist = true;
                        $scope.current.multiplehours = false;
                        break;
                    case '/api/v1/items/playlist/item':
                        $scope.current.playlist = true;
                        $scope.current.multiplehours = true;
                        break;
                }
            }
        }*/

        $scope.updateEndpoint = function(){
            if ($scope.current) {
                if ($scope.current.type == 'now-on-air') {
                    $scope.current.endpoint = '/api/v1/items/current/';
                    return;
                } else  {
                    if ($scope.current.keep_structure) {
                        $scope.current.endpoint = '/api/v1/items/playlist/keep_structure';
                    } else {
                        $scope.current.endpoint = '/api/v1/items/playlist/';
                    }
                    return;
                }
                
            }
        }

        /**
         * Keeps the currently selected file
         *
         * @var {Object} current
         */
        $scope.current = null;

        /**
         * Keeps the logs
         *
         * @var {Object}
         */
        $scope.logs = logs;

        $scope.logfiles = logfiles;

        /**
         * on log added push it to the right file
         *
         * @param {} key
         * @param {} log
         */
        ipcRenderer.on('logAdded', function(e, key, log) {
            $scope.logs[key].push(log);
            console.log(key, log);
        });

        /**
         * Keep the config files
         *
         * @var {Object} files
         */
        $scope.files = files;
        for (let key in $scope.files) $scope.files[key].currentKey = key;


        /**
         * Handles the click event when selecting a configuration file
         *
         * @param {String} key
         */
        $scope.onItemClick = function(key, focusOnID) {
            console.log(key);
            $scope.current = $scope.files[key];
            $scope.current.currentKey = key;
            $scope.current.currentKeyClean =  key.replace(/(\W|\s)/g, '_');
            
            //transform parts of the config to array of objects so the frontend can handle them
            
            if (typeof $scope.current.items != "undefined") {

                $scope.current.items.forEach( function(itemConfig) {
                    $scope.prepareItemConfig(itemConfig)
                });
                
                $scope.updateMatchingFields();
                
            } else {
                $scope.current.items = [];
                $scope.matchingFields = [];
            }

            $scope.refreshLogs($scope.current.currentKeyClean);
            
            
            if (focusOnID)
                document.getElementById('itemIdField').focus();
        };


        /**
         *  Prepare the Item config arrays
         */

        
        $scope.prepareItemConfig = function(itemConfig) {
            if (typeof itemConfig.data_field_xpaths != "undefined") {
                itemConfig.data_field_xpaths_array = Object.keys(itemConfig.data_field_xpaths).map(function (key) {
                    return {key: key, value: itemConfig.data_field_xpaths[key]};
                });
            } else {
                itemConfig.data_field_xpaths_array = [];
            }

            if (typeof itemConfig.data_field_indexes != "undefined") {
                itemConfig.data_field_indexes_array = Object.keys(itemConfig.data_field_indexes).map(function (key) {
                    return {key: key, value: itemConfig.data_field_indexes[key]};
                });
            } else {
                itemConfig.data_field_indexes_array = [];
            }

            if (typeof  itemConfig.data_field_enrichments != "undefined") {
                itemConfig.data_field_enrichments_array = Object.keys(itemConfig.data_field_enrichments).map(function (key) {
                    return {key: key, value: itemConfig.data_field_enrichments[key]};
                });
            } else {
                itemConfig.data_field_enrichments_array = [];
            }
            
            return itemConfig;
        }

        /**
         * Add a new model mapping row to the current selected file
         *
         */
        $scope.addItemConfig = function() {
            if (!$scope.current) return false;

            $scope.current.items.push({
                id: 0, 
                model_type_field: '', 
                model_type_match: '', 
                model_type_match_type: '',
                data_field_xpaths_array: [],
                data_field_indexes_array: [],
                data_field_enrichments_array: []});
        };

        $scope.duplicateItemConfig = function(itemId) {
            if (!$scope.current) return false;

            let config = angular.copy($scope.current.items[itemId]);
            config.id = "";
            config.model_type_match = "";
            
            $scope.current.items.push(config);

            let newId = $scope.current.items.length -1;
            
            $scope.updateDataObject(newId);
            $scope.updateDataEnrichments(newId);
            
            //we need to wait till the element is rendered to scroll to it
            //let's keep it simple and use a timeout
            let scrollTimeOut = setTimeout(
                () => {document.getElementById("item-config-"+newId).scrollIntoView(true);}
                , 300);
            
            
        };

        $scope.itemConfigToTop = function(itemId) {
            if (!$scope.current) return false;

            let config = $scope.current.items[itemId];
            $scope.current.items.splice(itemId, 1);
            $scope.current.items.unshift(config);
            
            document.getElementById("item-config-0").scrollIntoView(true);
            
        };



        /**
         * Add a new data selector row to an itemconfig
         *
         */
        $scope.addDataValue = function(itemId) {
            if (!$scope.current || !$scope.current.items[itemId] ) {
                return false;
            }
            if ($scope.current.input_format== "xml") {
                $scope.current.items[itemId].data_field_xpaths_array.push({key: '', value: ''});
            } else {
                $scope.current.items[itemId].data_field_indexes_array.push({key: '', value: ''});
            }
        };

 
        /**
         * Add a new data enrichment row to the current selected file
         *
         */
        $scope.addDataEnrichments = function(itemId) {
            if (!$scope.current || !$scope.current.items[itemId] ) {
                return false;
            }
            $scope.current.items[itemId].data_field_enrichments_array.push({key: '', value: ''});
        };
        
        
        
        
        /**
         * Maps the data_array to data object
         *
         */
        $scope.updateDataObject = function(itemId) {
            if (!$scope.current || !$scope.current.items[itemId] ) {sho
                return false;
            }
            var data = {};
            if ($scope.current.input_format== "xml") {
                $scope.current.items[itemId].data_field_xpaths_array.forEach(function (d) {
                    data[d.key] = d.value;
                });
                $scope.current.items[itemId].data_field_xpaths = data;
                $scope.updateMatchingFields();
            } else {
                $scope.current.items[itemId].data_field_indexes_array.forEach(function (d) {
                    data[d.key] = d.value;
                });
                $scope.current.items[itemId].data_field_indexes = data;      
            }
            
        };
        
        /**
         * Maps the data_enrichments_array to data object
         *
         */
        $scope.updateDataEnrichments = function(itemId) {
            if (!$scope.current || !$scope.current.items[itemId] ) {
                return false;
            }
            var data = {};
            $scope.current.items[itemId].data_field_enrichments_array.forEach(function(d) {
                data[d.key] = d.value;
            });

            $scope.current.items[itemId].data_field_enrichments = data;
        };


        $scope.deleteItemConfig = function(itemId) {
            if (!$scope.current || !$scope.current.items[itemId] ) {
                return false;
            }
           
            $scope.current.items.splice(itemId, 1);
            
        };
        
        $scope.deleteDataObject = function(itemId, key) {
            if (!$scope.current || !$scope.current.items[itemId] ) {
                return false;
            }
            if($scope.current.input_format== "xml" && $scope.current.items[itemId].data_field_xpaths_array[key]) {
                $scope.current.items[itemId].data_field_xpaths_array.splice(key, 1);
            }
            if($scope.current.input_format!= "xml" && $scope.current.items[itemId].data_field_indexes_array[key]) {
                $scope.current.items[itemId].data_field_indexes_array.splice(key, 1);
            }
            
            $scope.updateDataObject(itemId);
            
        };
        

        
        $scope.deleteDataEnrichment = function(itemId, key) {
            if (!$scope.current || !$scope.current.items[itemId] ) {
                return false;
            }
            if($scope.current.items[itemId].data_field_enrichments_array[key]) {
                $scope.current.items[itemId].data_field_enrichments_array.splice(key, 1);
            }
            $scope.updateDataEnrichments(itemId);
        };
        
        
        $scope.updateMatchingFields = function() {
            var oldMatches = [];
            var newMatches = [];
            var first = true;

            let fields = ($scope.current.input_format == "xml") ? "data_field_xpaths" : "data_field_indexes";
   
            $scope.current.items.forEach(function(itemConfig) {
                if (first &&  itemConfig[fields]) {
                    newMatches = Object.keys(itemConfig[fields]);
                    first = false;
                    return;
                }
                oldMatches = newMatches;
                newMatches = [];
                for (var key in itemConfig[fields]) {
                    if (oldMatches.indexOf(key) != -1) {
                        newMatches.push(key);
                    }
                }
                
            });
            
            $scope.matchingFields = newMatches;
            
        }


        $scope.addColumnAsc = function() {
            var largest = 1;
            if (typeof $scope.current.asc_column_width == "object" && 
                typeof $scope.current.asc_column_width[1] != 'undefined') {
                largest = Math.max.apply(Math, Object.keys($scope.current.asc_column_width)) +1;
            } else {
                $scope.current.asc_column_width = {};
            }

            $scope.current.asc_column_width[largest] =  0;
        }
        
        
        $scope.deleteColumnAsc = function(key) {
            delete $scope.current.asc_column_width[key];
            var newColumns = {};
            var i = 1;
            
            for (var key in $scope.current.asc_column_width) {
                newColumns[i++] = $scope.current.asc_column_width[key];
            }
            
            $scope.current.asc_column_width = newColumns;
        }
        
        
        /**
         * Saves all the configuration files
         *
         */
        $scope.save = function() {
            var files = {};
            var newCurrentKey;
            if ($scope.current) newCurrentKey = $scope.current.currentKey;
            
            Object.keys($scope.files).forEach(function(key) {
                var f = $scope.files[key];
                var newKey = (f.currentKey) ? f.currentKey : key;
                delete f["currentKey"];
                //delete f["data_array"];
                //delete f["model_type_mapping_array"];
                //delete f["data_enrichments_array"];
                f.items.forEach( function(itemConfig) {
                    delete itemConfig['data_field_xpaths_array'];
                    delete itemConfig['data_field_enrichments_array'];
                    delete itemConfig['data_field_indexes_array'];
                    delete itemConfig['$$hashKey'];
                });
                if (f.model_type_id)
                if (!Number.isNaN(Number(f.model_type_id)))
                    f.model_type_id = Number(f.model_type_id);
                files[newKey] = f;
            });

            $scope.files = files;
            for (let key in $scope.files) $scope.files[key].currentKey = key;

            writeConfig(files);

            ipcRenderer.send('configUpdated', files);
            
            if (newCurrentKey)
                $scope.onItemClick(newCurrentKey);
        };

        function writeConfig(files) {
            try {
                let dump = yaml.safeDump(files);
                fs.writeFileSync(configPath, dump, 'utf8');
                $mdToast.show(
                    $mdToast.simple()
                        .textContent('All configurations saved')
                        .position('top right')
                        .hideDelay(2000)
                );
                
            } catch (e) {

                $mdToast.show(
                    $mdToast.simple()
                        .textContent('Saving has failed! Contact an admin.')
                        .position('top right')
                        .hideDelay(2000)
                );
                
                console.log(e);
            }
        }

        $scope.importConfig = function() {
                var f = document.getElementById('importFile').files[0],
                    r = new FileReader();
                    r.onloadend = function(e){

                    var data = yaml.safeLoad(e.target.result);
                    var key = Object.keys(data)[0];
                    if ($scope.current) delete $scope.files[$scope.current.currentKey];

                    $scope.files[data[key].currentKey] = data[key];
                    $scope.$apply();

                    delete document.getElementById('importFile').files[0]
                    document.getElementById('config_entry_'+data[key].currentKey).click();


                }
                r.readAsBinaryString(f);
        }

        $scope.exportConfig = function () {
            if (!$scope.current || !$scope.current.currentKey) {
                alert("Nothing to export!");
                return;
            }

            dialog.showSaveDialog(function (fileName) {
                if (fileName === undefined){
                    console.log("You didn't save the file");
                    return;
                }

                var currentKey = $scope.current.currentKey;
                var currentConfig = $scope.files[currentKey];
                currentConfig['key'] = "YOUR_API_KEY_HERE";
                currentConfig['secret'] = "YOUR_API_SECRET_HERE";
                var content = yaml.safeDump({[currentKey] : currentConfig});

                // fileName is a string that contains the path and filename created in the save file dialog.
                fs.writeFile(fileName, content, function (err) {
                    if(err){
                        alert("An error ocurred creating the file "+ err.message);
                    }
                    alert("The file has been succesfully saved");
                });
            });
        }
        
        $scope.applyPreset = function (preset) {
            if (preset == "clear") {
                $scope.current = {'host' : 'radiomanager.io', 'port': '443'};
                return;
            }
            
            var dir = path.dirname(require.main.filename);
            dir = path.resolve(dir, '..', 'resources', 'presets');
            
            var contents = fs.readFileSync(dir+path.sep+preset+".yml");
            var data = yaml.safeLoad(contents);
            
            delete $scope.current.data;
            delete $scope.current.model_type_mapping;
            
            for (var key in data[0]) {
                for (var param in data[0][key]) {
                    $scope.current[param] = data[0][key][param];
                }
            }

            $scope.current.items.forEach( function(itemConfig) {
                $scope.prepareItemConfig(itemConfig)
            });

            $scope.updateMatchingFields();
            
            document.getElementById('config_entry_'+$scope.current.currentKeyClean).click();


            //console.log(f);

        }

        /**
         * Toggle active watcher
         *
         */
        $scope.setActive = function(key) {
            if (
                ($scope.files[key].useHttp && $scope.files[key].url) ||
                (!$scope.files[key].useHttp && $scope.files[key].path)
            ){
                ipcRenderer.send('configUpdateActive', key, $scope.files[key].active);
            } else {
                $scope.files[key].active = false;
            }
        };

        /**
         * Adds a new configuration file
         *
         */
        var n = 0;
        $scope.addFile = function() {
            let nId = 'New' + (n++);
            $scope.files[nId] = {
                host: 'https://radiomanager.io',
                endpoint: '/api/v1/items/current/',
                useHttp: false,
                url: '/',
                path: '',
                duration_ms: false,
                data: {
                    title: '',
                    duration: '',
                    external_id: '',
                },
                currentKeyClean: nId
            };

            $scope.onItemClick(nId, true);
        };

        /**
         * Deletes the currently selected configuration file
         *
         */
        $scope.deleteCurrent = function() {
            delete $scope.files[$scope.current.currentKey];
            $scope.current = null;
            $scope.save();
        };

        /**
         * Find key in data_array
         *
         */
        $scope.isKey = function (key) {
            if($scope.current && $scope.matchingFields){
                for (var i = 0, l = $scope.matchingFields.length; i < l; i++) {
                    if ($scope.matchingFields[i] == key) return true;
                }
            }
            return false;
        };

        /**
         * Guess the local timezone (just for placeholder in interface)
         */
        $scope.guessTimezone = function () {
            return moment.tz.guess();
        };
        
        
        $scope.selectLogFile = function(cleanKey, filename) {
            $scope.logs[cleanKey] = fs.readFileSync(filename, 'utf-8').split('\n');
        };
        
        
        $scope.refreshLogs = function(cleanKey) {
            try {
                $scope.logfiles[cleanKey] = globule.find(logsPath + "/" + cleanKey + "/*.log");

                $scope.logfiles[cleanKey].reverse()
                $scope.logfiles[cleanKey].slice(0, 30);

                let today = new moment();
                today = today.format('YYYY-MM-DD');
                let todaysLog = globule.find(logsPath + "/" + cleanKey + "/" + today + ".log");

                if (typeof todaysLog[0] != 'undefined') {
                    logs[cleanKey] = fs.readFileSync(todaysLog[0], 'utf-8').split('\n');
                    $scope.currentLog = todaysLog[0];
                } else {
                    $scope.logs[cleanKey] = [];
                    $scope.currentLog = null;
                }
            } catch (e) {
                $scope.logs[cleanKey] = [];
                $scope.currentLog = null;             
            }
        };
        
        /**
         * maybe not the best place, but select *something* when initializing
         */
        angular.element(document).ready(function () {
            var init_key = Object.keys($scope.files)[0];
            if (!$scope.files[init_key]) {
                $scope.addFile()
            } else {
                $scope.onItemClick(init_key, false);
            }

        });
    }

}());
